@@ -1,5 +1,7 @@
 /* Common declarations for the tar program.
-   Copyright 1988,92,93,94,96,97,99,2000, 2001 Free Software Foundation, Inc.
+
+   Copyright 1988, 1992, 1993, 1994, 1996, 1997, 1999, 2000, 2001 Free
+   Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -147,11 +149,8 @@ GLOBAL const char *use_compress_program_option;
 /* Boolean value.  */
 GLOBAL int dereference_option;
 
-/* Patterns with '/' that match file names to be excluded.  */
-GLOBAL struct exclude *excluded_with_slash;
-
-/* Patterns without '/' that match file names to be excluded.  */
-GLOBAL struct exclude *excluded_without_slash;
+/* Patterns that match file names to be excluded.  */
+GLOBAL struct exclude *excluded;
 
 /* Specified file containing names to work on.  */
 GLOBAL const char *files_from_option;
@@ -495,6 +494,7 @@ void chown_error_details PARAMS ((char const *, uid_t, gid_t));
 void close_error PARAMS ((char const *));
 void close_warn PARAMS ((char const *));
 void exec_fatal PARAMS ((char const *)) __attribute__ ((noreturn));
+void link_error PARAMS ((char const *, char const *));
 void mkdir_error PARAMS ((char const *));
 void mkfifo_error PARAMS ((char const *));
 void mknod_error PARAMS ((char const *));
@@ -559,7 +559,7 @@ char *name_from_list PARAMS ((void));
 void blank_name_list PARAMS ((void));
 char *new_name PARAMS ((const char *, const char *));
 
-int excluded_name PARAMS ((char const *));
+bool excluded_name PARAMS ((char const *));
 
 void add_avoided_name PARAMS ((char const *));
 int is_avoided_name PARAMS ((char const *));
