@@ -1,5 +1,8 @@
 /* Diff files from a tar archive.
-   Copyright 1988,92,93,94,96,97,99,2000, 2001 Free Software Foundation, Inc.
+
+   Copyright 1988, 1992, 1993, 1994, 1996, 1997, 1999, 2000, 2001 Free
+   Software Foundation, Inc.
+
    Written by John Gilmore, on 1987-04-30.
 
    This program is free software; you can redistribute it and/or modify it
@@ -133,7 +136,7 @@ process_dumpdir (size_t bytes, char *buffer)
    nonzero for success.  It it return error once, continue skipping
    without calling PROCESSOR anymore.  */
 static void
-read_and_process (size_t size, int (*processor) (size_t, char *))
+read_and_process (off_t size, int (*processor) (size_t, char *))
 {
   union block *data_block;
   size_t data_size;
