@@ -1,5 +1,7 @@
 /* Remote connection server.
-   Copyright 1994, 1995, 1996, 1997, 1999 Free Software Foundation, Inc.
+
+   Copyright (C) 1994, 1995, 1996, 1997, 1999, 2000, 2001 Free Software
+   Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -30,7 +32,10 @@
    MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  */
 
 #include "system.h"
-#include "safe-read.h"
+#include <copysym.h>
+#include <localedir.h>
+#include <safe-read.h>
+#include <full-write.h>
 
 #include <getopt.h>
 #include <sys/socket.h>
@@ -71,10 +76,7 @@ static FILE *debug_file;
 #define	DEBUG2(File, Arg1, Arg2) \
   if (debug_file) fprintf(debug_file, File, Arg1, Arg2)
 
-/*------------------------------------------------.
-| Return an error string, given an error number.  |
-`------------------------------------------------*/
-
+/* Return an error string, given an error number.  */
 #if HAVE_STRERROR
 # ifndef strerror
 char *strerror ();
@@ -93,10 +95,6 @@ private_strerror (int errnum)
 # define strerror private_strerror
 #endif
 
-/*---.
-| ?  |
-`---*/
-
 static void
 report_error_message (const char *string)
 {
@@ -106,10 +104,6 @@ report_error_message (const char *string)
   full_write (STDOUT_FILENO, reply_buffer, strlen (reply_buffer));
 }
 
-/*---.
-| ?  |
-`---*/
-
 static void
 report_numbered_error (int num)
 {
@@ -119,10 +113,6 @@ report_numbered_error (int num)
   full_write (STDOUT_FILENO, reply_buffer, strlen (reply_buffer));
 }
 
-/*---.
-| ?  |
-`---*/
-
 static void
 get_string (char *string)
 {
@@ -139,10 +129,6 @@ get_string (char *string)
   string[counter] = '\0';
 }
 
-/*---.
-| ?  |
-`---*/
-
 static void
 prepare_record_buffer (size_t size)
 {
@@ -283,10 +269,6 @@ Manipulate a tape drive, accepting commands from a remote process.\n\
   exit (status);
 }
 
-/*---.
-| ?  |
-`---*/
-
 int
 main (int argc, char *const *argv)
 {
@@ -311,12 +293,16 @@ main (int argc, char *const *argv)
       usage (EXIT_SUCCESS);
       
     case 'v':
-      printf ("rmt (GNU %s) %s\n%s\n%s\n", PACKAGE, VERSION,
-	      "Copyright 1999 Free Software Foundation, Inc.",
-	      _("\
+      {
+	char buf[MB_LEN_MAX + 1];
+	printf ("rmt (GNU %s) %s\n", PACKAGE, VERSION);
+	printf ("Copyright %s 2001 Free Software Foundation, Inc.\n",
+		copyright_symbol (buf, sizeof buf));
+	puts (_("\
 This program comes with NO WARRANTY, to the extent permitted by law.\n\
 You may redistribute it under the terms of the GNU General Public License;\n\
 see the file named COPYING for details."));
+      }
       return EXIT_SUCCESS;
 
     case -1:
